// ReadHeader.c		All of the header reading routines. Spectral and sound.
#if powerc
#include <fp.h>
#endif	/*powerc*/

#include "IEEE80.h"
#include "SoundFileHeader.h"
#include "OpenSoundFile.h"
#include "ReadHeader.h"
#include "Main.h"
#include "Macros.h"
#include "Misc.h"
#include "ByteSwap.h"
#include "Dialog.h"
#include "AppleCompression.h"
#include <stdio.h>

/* globals */
extern GlowBalls	global;

short
ReadSDIIHeader(SoundInfo *mySI)
{
	long		intSRate;
	Handle		frameSizeHandle, sRateHandle, nChansHandle;
	Str255		tmpStr;
	
	mySI->rFileNum = FSpOpenResFile(&(mySI->sfSpec),fsCurPerm);
	if(mySI->rFileNum == -1)
		return(-1);
	UseResFile(mySI->rFileNum);
	frameSizeHandle = GetNamedResource('STR ',"\psample-size");
	if(frameSizeHandle==0)
		return(-1);
	sRateHandle = GetNamedResource('STR ',"\psample-rate");
	if(sRateHandle==0)
		return(-1);
	nChansHandle = GetNamedResource('STR ',"\pchannels");
	if(nChansHandle==0)
		return(-1);
		
	HLock(nChansHandle);
	StringCopy((StringPtr)*nChansHandle, tmpStr);
	StringToNum(tmpStr, &mySI->nChans);
	HUnlock(nChansHandle);

	HLock(sRateHandle);
	StringCopy((StringPtr)*sRateHandle, tmpStr);
	if(mySI->sfType == DSPD)
	{
		StringToNum(tmpStr, &intSRate);
		mySI->sRate = (float)intSRate;
	}
	else
		StringToFix(tmpStr, &mySI->sRate);
	HUnlock(sRateHandle);

	HLock(frameSizeHandle);
	StringCopy((StringPtr)*frameSizeHandle, tmpStr);
	StringToFix(tmpStr, &mySI->frameSize);
	HUnlock(frameSizeHandle);
	
	CloseResFile(mySI->rFileNum);
	ReleaseResource(frameSizeHandle);
	ReleaseResource(sRateHandle);
	ReleaseResource(nChansHandle);
	
	UseResFile(global.appFileNum);
	mySI->dataStart = 0L;
	if(mySI->frameSize == 4.0)
		mySI->packMode = SF_FORMAT_32_LINEAR;
	else if(mySI->frameSize == 3.0)
		mySI->packMode = SF_FORMAT_24_LINEAR;
	else if(mySI->frameSize == 2.0)
		mySI->packMode = SF_FORMAT_16_LINEAR;
	else
		mySI->packMode = SF_FORMAT_8_LINEAR;
	GetEOF(mySI->dFileNum, &mySI->numBytes);
	mySI->dataEnd = mySI->numBytes;
	return(TRUE);
}

short
ReadAIFFHeader(SoundInfo *mySI)
{
	short	soundFound = FALSE, commonFound = FALSE, peakFound = FALSE;
	long	readSize, filePos;
	OSErr	error;
	FormChunk		myFormChunk;
	AIFFCommonChunk		myCommonChunk;
	SoundDataChunkHeader	mySoundDataChunk;
	PeakChunkHeader	myPeakChunkHeader;
	PositionPeak aPositionPeak;
	int i;
	
	struct
	{
		long	ckID;
		long	ckSize;
	}	chunkHeader;
		
	SetFPos(mySI->dFileNum, fsFromStart, 0L);
	readSize = sizeof(myFormChunk);
	FSRead(mySI->dFileNum, &readSize, &myFormChunk);
	if(myFormChunk.ckID != FORMID)
		return(-1);
	else if(myFormChunk.formType != AIFFTYPE)
		return(-1);
	/* Loop through the Chunks until finding the Common Chunk and Sound Chunk */
	while(commonFound == FALSE || soundFound == FALSE || peakFound == FALSE)
	{
		readSize = sizeof(chunkHeader);
		error = FSRead(mySI->dFileNum, &readSize, &chunkHeader);
		if((error == eofErr) && commonFound && soundFound)
		{
			if(peakFound == FALSE)
				mySI->peak = mySI->peakFL = mySI->peakFR = mySI->peakRL = mySI->peakRR = 1.0;
			return(TRUE);
		}
		else if((error == eofErr) && (commonFound == FALSE || soundFound == FALSE))
			return(-1);
			
		switch(chunkHeader.ckID)
		{
			case COMMONID:
				readSize = sizeof(myCommonChunk);
				SetFPos(mySI->dFileNum, fsFromMark, -(sizeof(chunkHeader)));
				FSRead(mySI->dFileNum, &readSize, &myCommonChunk);
				mySI->sRate = ieee_80_to_double(myCommonChunk.sampleRate);
				mySI->nChans = myCommonChunk.numChannels;
				switch(myCommonChunk.sampleSize)
				{
					case AIFF_FORMAT_LINEAR_8:
						mySI->frameSize = 1;
						mySI->packMode = SF_FORMAT_8_LINEAR;
						break;
					case AIFF_FORMAT_LINEAR_16:
						mySI->frameSize = 2;
						mySI->packMode = SF_FORMAT_16_LINEAR;
						break;
					case AIFF_FORMAT_LINEAR_24:
						mySI->frameSize = 3;
						mySI->packMode = SF_FORMAT_24_LINEAR;
						break;
					case AIFF_FORMAT_LINEAR_32:
						mySI->frameSize = 4;
						mySI->packMode = SF_FORMAT_32_LINEAR;
						break;
				}
				mySI->numBytes = (long)(myCommonChunk.numSampleFrames
					*mySI->frameSize*mySI->nChans);
				commonFound = TRUE;
				break;
			case SOUNDID:
				// set filePos to point just after chunkHeader
				GetFPos(mySI->dFileNum, &filePos);
				// back up before chunkHeader
				SetFPos(mySI->dFileNum, fsFromMark, -(sizeof(chunkHeader)));
				// grab the offset and chunksize put in sound struct.
				readSize = sizeof(SoundDataChunkHeader);
				FSRead(mySI->dFileNum, &readSize, &mySoundDataChunk);
				mySI->dataStart = filePos + mySoundDataChunk.offset + 8;
				// the end of chunk is ckSize after the chunkHeader
				filePos += chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromStart, filePos);
				mySI->dataEnd = filePos;
				soundFound = TRUE;
				break;
			case PEAKID:
				// set filePos to point just after chunkHeader
				GetFPos(mySI->dFileNum, &filePos);
				if(commonFound == TRUE)
				{
					mySI->peak = 0.0;
					readSize = sizeof(PeakChunkHeader);
					SetFPos(mySI->dFileNum, fsFromMark, -(sizeof(chunkHeader)));
					FSRead(mySI->dFileNum, &readSize, &myPeakChunkHeader);
					for(i = 0; i < myCommonChunk.numChannels; i++)
					{
						readSize = sizeof(PositionPeak);
						FSRead(mySI->dFileNum, &readSize, &aPositionPeak);
						switch(i)
						{
							case 0:
								mySI->peakFL = aPositionPeak.value;
								break;
							case 1:
								mySI->peakFR = aPositionPeak.value;
								break;
							case 2:
								mySI->peakRL = aPositionPeak.value;
								break;
							case 3:
								mySI->peakRR = aPositionPeak.value;
								break;
						}
						if(aPositionPeak.value > mySI->peak)
							mySI->peak = aPositionPeak.value;
					}
				}
				filePos += chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromStart, filePos);
				peakFound = TRUE;
				break;
			case NAMEID:
			{
				char testChar;
				GetFPos(mySI->dFileNum, &filePos);
				readSize = sizeof(char);
				FSRead(mySI->dFileNum, &readSize, &testChar);
				if(testChar == '|')
				mySI->packMode = SF_FORMAT_3DO_CONTENT;
				filePos += chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromStart, filePos);
				break;
			}
			case MARKERID:
			case INSTRUMENTID:
			case MIDIDATAID:
			case AUDIORECORDINGID:
			case APPLICATIONSPECIFICID:
			case COMMENTID:
			case AUTHORID:
			case COPYRIGHTID:
			case ANNOTATIONID:
			case FORMVERID:
			case JUNKID:
			default:
				filePos = chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromMark, filePos);
				break;
		}

	}

	return(TRUE);
}

short
ReadAIFCHeader(SoundInfo *mySI)
{
	short				soundFound = FALSE, commonFound = FALSE, nameFound = FALSE;
	Boolean				appSpecNeeded = TRUE, foulSE16stench = FALSE, peakFound = FALSE;
	long				readSize, filePos, i, tmpLong;
	FormChunk			myFormChunk;
	FormatVersionChunk	myFormatVersionChunk;
	AIFCCommonChunk		myCommonChunk;
	SoundDataChunkHeader		mySoundDataChunk;
	PeakChunkHeader		myPeakChunkHeader;
	PositionPeak		aPositionPeak;
	OSType				myOSType;
	OSErr				error;

	struct
	{
		long	ckID;
		long	ckSize;
	}	chunkHeader;
	SetFPos(mySI->dFileNum, fsFromStart, 0L);
	readSize = sizeof(myFormChunk);
	FSRead(mySI->dFileNum, &readSize, &myFormChunk);
	if(myFormChunk.ckID != FORMID)
		return(-1);
	else if(myFormChunk.formType != AIFCTYPE)
		return(-1);
	/* Loop through the Chunks until finding the Common Chunk and Sound Chunk */
	while(commonFound == FALSE || soundFound == FALSE || appSpecNeeded == TRUE || peakFound == FALSE)
	{
		readSize = sizeof(chunkHeader);
		error = FSRead(mySI->dFileNum, &readSize, &chunkHeader);
		if((error == eofErr) && commonFound && soundFound)
		{
			if(peakFound == FALSE)
				mySI->peak = mySI->peakFL = mySI->peakFR = mySI->peakRL = mySI->peakRR = 1.0;
			return(TRUE);
		}
		else if((error == eofErr) && (commonFound == FALSE || soundFound == FALSE))
			return(-1);
			
		switch(chunkHeader.ckID)
		{
			case FORMVERID:
				readSize = sizeof(myFormatVersionChunk);
				SetFPos(mySI->dFileNum, fsFromMark, -(sizeof(chunkHeader)));
				FSRead(mySI->dFileNum, &readSize, &myFormatVersionChunk);
				if(myFormatVersionChunk.timestamp != AIFCVERSION1)
				{
					DrawErrorMessage("\pUnsupported AIFC format");
					return(-1);
				}
				break;
			case COMMONID:
				readSize = sizeof(myCommonChunk);
				GetFPos(mySI->dFileNum, &filePos);
				SetFPos(mySI->dFileNum, fsFromMark, -(sizeof(chunkHeader)));
				FSRead(mySI->dFileNum, &readSize, &myCommonChunk);
				mySI->sRate = ieee_80_to_double(myCommonChunk.sampleRate);
				mySI->nChans = myCommonChunk.numChannels;
				for(i=0; i<=myCommonChunk.compressionName[0]; i++)
					mySI->compName[i] = myCommonChunk.compressionName[i];
				switch(myCommonChunk.compressionType)
				{
					case kIMACompression:
						if(global.soundCompress == FALSE)
						{
							return(-1);
						}
						else
						{
							mySI->frameSize = 0.53125;
							mySI->packMode = SF_FORMAT_4_ADIMA;
						}
						break;
					case kMACE3Compression:
						if(global.soundCompress == FALSE)
						{
							return(-1);
						}
						else
						{
							mySI->frameSize = 1.0/3.0;
							mySI->packMode = SF_FORMAT_MACE3;
						}
						break;
					case kMACE6Compression:
						if(global.soundCompress == FALSE)
						{
							return(-1);
						}
						else
						{
							mySI->frameSize = 1.0/6.0;
							mySI->packMode = SF_FORMAT_MACE6;
						}
						break;
					case k8BitOffsetBinaryFormat:
						mySI->frameSize = 1.0;
						mySI->packMode = SF_FORMAT_8_UNSIGNED;
						break;
					case k16BitBigEndianFormat:
						mySI->frameSize = 2.0;
						mySI->packMode = SF_FORMAT_16_LINEAR;
						break;
					case k16BitLittleEndianFormat:
						mySI->frameSize = 2.0;
						mySI->packMode = SF_FORMAT_16_SWAP;
					case AIFC_ID_ADDVI:
						mySI->frameSize = 0.5;
						mySI->packMode = SF_FORMAT_4_ADDVI;
						break;
					case kULawCompression:
						mySI->frameSize = 1.0;
						mySI->packMode = SF_FORMAT_8_MULAW;
						break;
					case kALawCompression:
						mySI->frameSize = 1.0;
						mySI->packMode = SF_FORMAT_8_ALAW;
						break;
					case k24BitFormat:
						mySI->frameSize = 3.0;
						mySI->packMode = SF_FORMAT_24_COMP;
						break;
					case k32BitFormat:
						mySI->frameSize = 4.0;
						mySI->packMode = SF_FORMAT_32_COMP;
						break;
					case AIFC_ID_FLT32:
					case kFloat32Format:
						mySI->frameSize = 4.0;
						mySI->packMode = SF_FORMAT_32_FLOAT;
						break;
					case kFloat64Format:
					case k32BitLittleEndianFormat:
					case kCDXA4Compression:
					case kCDXA2Compression:
					case kMicrosoftADPCMFormat:
					case kDVIIntelIMAFormat:
					case kDVAudioFormat:
					case kQDesignCompression:
					case kQUALCOMMCompression:
					case kMPEGLayer3Format:
					case kFullMPEGLay3Format:
						DrawErrorMessage("\pSoundHack can't read this type of compression");
						return(-1);
						break;
					case kSoundNotCompressed:
						switch(myCommonChunk.sampleSize)
						{
							case AIFF_FORMAT_LINEAR_8:
								mySI->frameSize = 1.0;
								mySI->packMode = SF_FORMAT_8_LINEAR;
								break;
							case AIFF_FORMAT_LINEAR_16:
								mySI->frameSize = 2.0;
								mySI->packMode = SF_FORMAT_16_LINEAR;
								break;
							case AIFF_FORMAT_LINEAR_24:
								mySI->frameSize = 3.0;
								mySI->packMode = SF_FORMAT_24_LINEAR;
								break;
							case AIFF_FORMAT_LINEAR_32:
								mySI->frameSize = 4.0;
								mySI->packMode = SF_FORMAT_32_LINEAR;
								break;
						}
						break;
					}
				if(mySI->packMode != SF_FORMAT_32_FLOAT)
					appSpecNeeded = FALSE;
				tmpLong = (long)(myCommonChunk.numSampleFrames * mySI->frameSize * mySI->nChans);
				if(mySI->packMode == SF_FORMAT_4_ADIMA && ((float)tmpLong/(float)(myFormChunk.ckSize) < .95))
				{
					foulSE16stench = TRUE;
					mySI->numBytes = (long)(myCommonChunk.numSampleFrames * 34 * mySI->nChans);
				}
				else
					mySI->numBytes = tmpLong;
				commonFound = TRUE;
				filePos += chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromStart, filePos);
				break;
			case SOUNDID:
				// set filePos to point just after chunkHeader
				GetFPos(mySI->dFileNum, &filePos);
				// back up before chunkHeader
				SetFPos(mySI->dFileNum, fsFromMark, -(sizeof(chunkHeader)));
				// grab the offset and chunksize put in sound struct.
				readSize = sizeof(SoundDataChunkHeader);
				FSRead(mySI->dFileNum, &readSize, &mySoundDataChunk);
				mySI->dataStart = filePos + mySoundDataChunk.offset + 8;
				// the end of chunk is ckSize after the chunkHeader
				filePos += chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromStart, filePos);
				mySI->dataEnd = filePos;
				soundFound = TRUE;
				break;
			case PEAKID:
				// set filePos to point just after chunkHeader
				GetFPos(mySI->dFileNum, &filePos);
				if(commonFound == TRUE)
				{
					mySI->peak = 0.0;
					readSize = sizeof(PeakChunkHeader);
					SetFPos(mySI->dFileNum, fsFromMark, -(sizeof(chunkHeader)));
					FSRead(mySI->dFileNum, &readSize, &myPeakChunkHeader);
					for(i = 0; i < myCommonChunk.numChannels; i++)
					{
						readSize = sizeof(PositionPeak);
						FSRead(mySI->dFileNum, &readSize, &aPositionPeak);
						switch(i)
						{
							case 0:
								mySI->peakFL = aPositionPeak.value;
								break;
							case 1:
								mySI->peakFR = aPositionPeak.value;
								break;
							case 2:
								mySI->peakRL = aPositionPeak.value;
								break;
							case 3:
								mySI->peakRR = aPositionPeak.value;
								break;
						}
						if(aPositionPeak.value > mySI->peak)
							mySI->peak = aPositionPeak.value;
					}
				}
				filePos += chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromStart, filePos);
				peakFound = TRUE;
				break;
			case APPLICATIONSPECIFICID:
				// set filePos to point just after chunkHeader
				GetFPos(mySI->dFileNum, &filePos);
				// see if the OSTypeID is right for gain values
				readSize = sizeof(myOSType);
				FSRead(mySI->dFileNum, &readSize, &myOSType);
				if(myOSType == 'pErF')
				{
					if(chunkHeader.ckSize >= 8)
					{
						readSize = sizeof(float);
						FSRead(mySI->dFileNum, &readSize, &mySI->peak);
					}
					if(chunkHeader.ckSize >= 12)
					{
						readSize = sizeof(float);
						FSRead(mySI->dFileNum, &readSize, &mySI->peakFL);
					}
					if(chunkHeader.ckSize >= 16)
					{
						readSize = sizeof(float);
						FSRead(mySI->dFileNum, &readSize, &mySI->peakFR);
					}
					if(chunkHeader.ckSize >= 20)
					{
						readSize = sizeof(float);
						FSRead(mySI->dFileNum, &readSize, &mySI->peakRL);
					}
					if(chunkHeader.ckSize >= 24)
					{
						readSize = sizeof(float);
						FSRead(mySI->dFileNum, &readSize, &mySI->peakRR);
					}
				}
				
				// the end of chunk is ckSize after the chunkHeader
				filePos += chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromStart, filePos);
				appSpecNeeded = FALSE;
				break;
			case NAMEID:
			{
				char testChar;
				GetFPos(mySI->dFileNum, &filePos);
				readSize = sizeof(char);
				FSRead(mySI->dFileNum, &readSize, &testChar);
				if(testChar == '|')
				mySI->packMode = SF_FORMAT_3DO_CONTENT;
				filePos += chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromStart, filePos);
				break;
			}
			case MARKERID:
			case INSTRUMENTID:
			case MIDIDATAID:
			case AUDIORECORDINGID:
			case COMMENTID:
			case AUTHORID:
			case COPYRIGHTID:
			case ANNOTATIONID:
			case JUNKID:
			default:
				filePos = chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromMark, filePos);
				break;
		}
	}
	return(TRUE);
}


short
ReadWAVEHeader(SoundInfo *mySI)
{
	short					soundFound = FALSE, commonFound = FALSE;
	short				sampleSize;
	long				readSize, filePos;
	RIFFFormChunk		myFormChunk;
	WAVFormatChunk		myFormatChunk;
	
	struct
	{
		long	ckID;
		long	ckSize;
	}	chunkHeader;
	SetFPos(mySI->dFileNum, fsFromStart, 0L);
	readSize = sizeof(myFormChunk);
	FSRead(mySI->dFileNum, &readSize, &myFormChunk);
	if(myFormChunk.ckID != WAV_ID_RIFF)
		return(-1);
	else if(myFormChunk.formType != WAV_ID_WAVE)
		return(-1);
	/* Loop through the Chunks until finding the Format Chunk and Sound Chunk */
	while(commonFound == FALSE || soundFound == FALSE)
	{
		readSize = sizeof(chunkHeader);
		FSRead(mySI->dFileNum, &readSize, &chunkHeader);
		switch(chunkHeader.ckID)
		{
			case WAV_ID_FORMAT:
				readSize = sizeof(myFormatChunk);
				GetFPos(mySI->dFileNum, &filePos);
				SetFPos(mySI->dFileNum, fsFromMark, -(sizeof(chunkHeader)));
				FSRead(mySI->dFileNum, &readSize, &myFormatChunk);
				chunkHeader.ckSize = myFormatChunk.ckSize = ByteSwapLong(myFormatChunk.ckSize);
				myFormatChunk.wFormatTag = ByteSwapShort(myFormatChunk.wFormatTag);
				myFormatChunk.wChannels = ByteSwapShort(myFormatChunk.wChannels);
				myFormatChunk.dwSamplePerSec = ByteSwapLong(myFormatChunk.dwSamplePerSec);
				myFormatChunk.dwAvgBytesPerSec = ByteSwapLong(myFormatChunk.dwAvgBytesPerSec);
				myFormatChunk.wBlockAlign = ByteSwapShort(myFormatChunk.wBlockAlign);
				mySI->sRate = (float)myFormatChunk.dwSamplePerSec;
				mySI->nChans = myFormatChunk.wChannels;
				switch(myFormatChunk.wFormatTag)
				{
					case WAV_FORMAT_PCM:
						readSize = 2;
						FSRead(mySI->dFileNum, &readSize, &sampleSize);
						mySI->frameSize = ByteSwapShort(sampleSize)/8.0;
						if(mySI->frameSize == 1)
							mySI->packMode = SF_FORMAT_8_UNSIGNED;
						else if(mySI->frameSize == 2)
							mySI->packMode = SF_FORMAT_16_SWAP;
						else if(mySI->frameSize == 3)
							mySI->packMode = SF_FORMAT_24_SWAP;
						else if(mySI->frameSize == 4)
							mySI->packMode = SF_FORMAT_32_SWAP;
						break;
						break;
					case IBM_FORMAT_MULAW:
						mySI->frameSize = 1;
						mySI->packMode = SF_FORMAT_8_MULAW;
						break;
					case IBM_FORMAT_ALAW:
						mySI->frameSize = 1;
						mySI->packMode = SF_FORMAT_8_ALAW;
					case IBM_FORMAT_ADDVI:
					case WAV_FORMAT_ADDVI:
					case WAV_FORMAT_ADIMA:
					default:
//						DrawErrorMessage("\pUnknown data format. Only 8 and 16 bit PCM format WAVE files supported.");
						return(-1);
						break;
				}
				commonFound = TRUE;
				SetFPos(mySI->dFileNum, fsFromStart, filePos+chunkHeader.ckSize);
				break;
			case WAV_ID_DATA:
				/* We only want to get the offset for start of sound and the number of bytes */
				mySI->numBytes = chunkHeader.ckSize = ByteSwapLong(chunkHeader.ckSize);
				GetFPos(mySI->dFileNum, &filePos);
				mySI->dataStart = filePos;
				filePos = chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromMark, filePos);
				mySI->dataEnd = filePos;
				soundFound = TRUE;
				break;
			default:
				chunkHeader.ckSize = ByteSwapLong(chunkHeader.ckSize);
				filePos = chunkHeader.ckSize;
				EVENUP(filePos);
				SetFPos(mySI->dFileNum, fsFromMark, filePos);
				break;
		}

	}

	return(TRUE);
}

/* NeXT and Sun have the same header */
short	ReadNEXTHeader(SoundInfo *mySI)
{
	Str255			errStr;
	char			*info;
	long			dFileEOF, readSize, theFilePos;
	float			peak;
	NeXTSoundInfo	myNeXTSI;
	
	SetFPos(mySI->dFileNum, fsFromStart, 0L);
	readSize = sizeof(myNeXTSI);
	FSRead(mySI->dFileNum, &readSize, &myNeXTSI);
	if(myNeXTSI.magic != NEXTMAGIC)
		return(-1);
	switch(myNeXTSI.dataFormat)
	{
		case NEXT_FORMAT_LINEAR_8:
			mySI->packMode = SF_FORMAT_8_LINEAR;
			mySI->frameSize = 1;
			break;
		case NEXT_FORMAT_LINEAR_16:
			mySI->packMode = SF_FORMAT_16_LINEAR;
			mySI->frameSize = 2;
			break;
		case NEXT_FORMAT_LINEAR_24:
			mySI->frameSize = 3;
			mySI->packMode = SF_FORMAT_24_LINEAR;
			break;
		case NEXT_FORMAT_LINEAR_32:
			mySI->frameSize = 4;
			mySI->packMode = SF_FORMAT_32_LINEAR;
			break;
		case NEXT_FORMAT_FLOAT:
			mySI->packMode = SF_FORMAT_32_FLOAT;
			mySI->frameSize = 4;
			break;
		case NEXT_FORMAT_MULAW_8:
			mySI->packMode = SF_FORMAT_8_MULAW;
			mySI->frameSize = 1;
			break;
		case NEXT_FORMAT_ALAW_8:
			mySI->packMode = SF_FORMAT_8_ALAW;
			mySI->frameSize = 1;
			break;
		default:
			DrawErrorMessage("\pUnrecognized packmode");
			NumToString(myNeXTSI.dataFormat, errStr);
			DrawErrorMessage(errStr);
			return(-1);
	}
	mySI->sRate = (float)myNeXTSI.samplingRate;
	mySI->nChans = myNeXTSI.channelCount;
	mySI->dataStart = myNeXTSI.dataLocation;
	GetEOF(mySI->dFileNum, &dFileEOF);
	mySI->numBytes = dFileEOF - myNeXTSI.dataLocation;
	mySI->dataEnd = dFileEOF;
	
	// read in the info string
	GetFPos(mySI->dFileNum, &theFilePos);
	readSize = myNeXTSI.dataLocation - theFilePos;
	info = (char *)NewPtr(readSize);
	FSRead(mySI->dFileNum, &readSize, info);
	if(info[0] != 0)
	{
		if(info[0] == 'P')
		{
			if(sscanf(info, "Peak %f", &peak))
				mySI->peak = mySI->peakFL = mySI->peakFR = mySI->peakRL = mySI->peakRR = peak;
			else
				mySI->peak = mySI->peakFL = mySI->peakFR = mySI->peakRL = mySI->peakRR = 1.0;
		}
	}
	DisposePtr((Ptr)info);
	return(TRUE);
}
	
short
ReadBICSFHeader(SoundInfo *mySI)
{
	long			dFileEOF, readSize;
	Boolean			codeLeft;
	BICSFSoundInfo	myBICSFSI;
	BICSFCodeHeader myCodeHeader;
	short			codePosition;
	
	SetFPos(mySI->dFileNum, fsFromStart, 0L);
	readSize = sizeof(myBICSFSI);
	FSRead(mySI->dFileNum, &readSize, &myBICSFSI);
	if(myBICSFSI.magic != BICSFMAGIC)
		return(-1);
	switch(myBICSFSI.packMode)
	{
		case BICSF_FORMAT_LINEAR_8:
			mySI->packMode = SF_FORMAT_8_LINEAR;
			break;
		case BICSF_FORMAT_LINEAR_16:
			mySI->packMode = SF_FORMAT_16_LINEAR;
			break;
		case BICSF_FORMAT_FLOAT:
			mySI->packMode = SF_FORMAT_32_FLOAT;
			break;
		default:
			DrawErrorMessage("\pUnrecognized pack mode.");
			return(-1);
	}
	mySI->sRate = myBICSFSI.srate;
	mySI->nChans = myBICSFSI.chans;
	mySI->frameSize = (double)myBICSFSI.packMode;
	mySI->dataStart = sizeof(myBICSFSI);
	GetEOF(mySI->dFileNum, &dFileEOF);
	mySI->numBytes = dFileEOF - sizeof(myBICSFSI);
	mySI->dataEnd = dFileEOF;

	// we are gonna look for some amp codes if float
	codeLeft = TRUE;
	codePosition = 0;
	if(myBICSFSI.packMode == BICSF_FORMAT_FLOAT)
		while(codeLeft)
		{
			BlockMoveData(&myBICSFSI.codes[codePosition], &myCodeHeader, sizeof(myCodeHeader));
			codePosition += sizeof(myCodeHeader);
			if(myCodeHeader.code == BICSF_CODE_END)
				break;
			else if(myCodeHeader.code == BICSF_CODE_MAXAMP)
			{
				BlockMoveData(&myBICSFSI.codes[codePosition], &mySI->peakFL, sizeof(float));
				codePosition += sizeof(float);
				mySI->peak = mySI->peakFL;
				if(mySI->nChans > 1)
				{
					BlockMoveData(&myBICSFSI.codes[codePosition], &mySI->peakFR, sizeof(float));
					codePosition += sizeof(float);
					if(mySI->peakFR > mySI->peak)
						mySI->peak = mySI->peakFR;
				}
				if(mySI->nChans > 2)
				{
					BlockMoveData(&myBICSFSI.codes[codePosition], &mySI->peakRL, sizeof(float));
					codePosition += sizeof(float);
					if(mySI->peakRL > mySI->peak)
						mySI->peak = mySI->peakRL;
				}
				if(mySI->nChans > 1)
				{
					BlockMoveData(&myBICSFSI.codes[codePosition], &mySI->peakRR, sizeof(float));
					codePosition += sizeof(float);
					if(mySI->peakRR > mySI->peak)
						mySI->peak = mySI->peakRR;
				}
				break;
			}
			else
			{
				codePosition += myCodeHeader.bsize;
				if(codePosition >= 1008)
					codeLeft = FALSE;
			}
		}
			
				
	return(TRUE);
}
// using quicktime to gather sound information
short ReadMPEGHeader(SoundInfo *mySI)
{
	SoundDescriptionV1Handle sourceSoundDescription;
	AudioFormatAtomPtr outAudioAtom;
	FSSpec tmpSpec;
	long dFileEOF;
	OSErr err;
	
	tmpSpec.parID = mySI->sfSpec.parID;
	tmpSpec.vRefNum = mySI->sfSpec.vRefNum;
	StringCopy(mySI->sfSpec.name, tmpSpec.name);

	if(FindSFType(mySI->sfSpec) != MPEG)
		return(-1);
	sourceSoundDescription = (SoundDescriptionV1Handle)NewHandle(0);
	err = MyGetSoundDescription(tmpSpec, sourceSoundDescription, &outAudioAtom, &mySI->timeScale);
	mySI->peak = mySI->peakFL = mySI->peakFR = mySI->peakRL = mySI->peakRR = 1.0;
	mySI->sRate = (*sourceSoundDescription)->desc.sampleRate >> 16;
	mySI->nChans = (*sourceSoundDescription)->desc.numChannels;
	mySI->frameSize = (float)(*sourceSoundDescription)->bytesPerPacket/(*sourceSoundDescription)->samplesPerPacket;
	mySI->dataStart = 0L;
	GetEOF(mySI->dFileNum, &dFileEOF);
	mySI->dataEnd = dFileEOF;
	mySI->numBytes = mySI->dataEnd - mySI->dataStart;
	mySI->packMode = SF_FORMAT_MPEG_III;
	DisposeHandle((Handle)sourceSoundDescription);
	DisposePtr((Ptr)outAudioAtom);
	if(err == noErr)
		return(TRUE);
	else 
		return(-1);

}
