/*
    RotateString sample code
    By Randy Theland and Brigham Stevens
    Apple Computer Developer Technical Support

    This function will return a BitMap that contains the
    text passed rotated 90

    The font characteristics are copied from
    the current port before creating the BitMap.

*/
#include <math.h>
#include "RotateString.h"

/*
    In pascal this is:

 function RotateString (str: Str255; var destMap: BitMap, direction : integer): OSErr;

*/

pascal OSErr RotateString( Str255 str, BitMap *destMap, short direction)
{
    GrafPort    srcPort;
    char        *srcPtr, *destPtr;
    FontInfo    f;
    unsigned short       height, width, strWidth, rotWidth, rotHeight;
    unsigned short		heightBytes, widthBytes, rotWidthBytes, rotHeightBytes;
    unsigned long		row, column;
    short       texFont,texMode,texSize;
    Style		texFace;
    long       err;
    GrafPtr     savePort;

    GetPort(&savePort);

    texFont = (qd.thePort)->txFont;
    texFace = (qd.thePort)->txFace;
    texMode = (qd.thePort)->txMode;
    texSize = (qd.thePort)->txSize;

    OpenPort(&srcPort);
    SetPort(&srcPort);

    TextFont(texFont);
    TextFace(texFace);
    TextMode(texMode);
    TextSize(texSize);
    GetFontInfo(&f);

    height = f.ascent + f.descent + f.leading;
    heightBytes = ((height + 7) & 0x7FF8) >> 3;
    strWidth = StringWidth(str);
    if(strWidth < 720)
    	width = 720;
    else
    	width = strWidth;
    widthBytes = ((width + 7) & 0x7FF8) >> 3;

    rotWidthBytes = heightBytes;
    rotWidth = rotWidthBytes << 3;
    rotHeight = widthBytes << 3;

    srcPort.portBits.baseAddr = NewPtrClear((long)(widthBytes * height));
    if(err = MemError()) return err;

    srcPort.portBits.rowBytes = widthBytes;
    SetRect(&srcPort.portBits.bounds, 0, 0, width-1, height-1);

    destMap->baseAddr = NewPtrClear((long)(rotWidthBytes * rotHeight));
    if(err = MemError()) 
    {
        DisposePtr(srcPort.portBits.baseAddr);
        return err;
    }

    destMap->rowBytes = rotWidthBytes;
    SetRect(&destMap->bounds, 0, 0, rotWidth-1, rotHeight-1);

    RectRgn(srcPort.visRgn,&srcPort.portBits.bounds);
    TextSize(texSize - 2);
    MoveTo( ((width - strWidth)>>1), f.ascent+1);  // change 2.0.1 <CKH> was height
    DrawString( str);


    /*****************************************/
    /*  Rotate Algorithm                     */
    /*****************************************/

    srcPtr = srcPort.portBits.baseAddr;     // Start at the Top,Left

	// Start at the Top,Right
	destPtr = destMap->baseAddr + (long)(rotWidthBytes - 1);

    for( column = 0; column < width; column++)       /* width is in pixels */
    {
        for( row = 0; row < height; row++)       /* height in pixels, too */
        {
			/*  We are rotating right (clock-wise) thus */
			/*  as we search across the top of the horizontal pixel bit map,    */
			/*  we write down the left edge of the vertical pixel bit map */

			if( *(srcPtr + (row*widthBytes) + (column >> 3)) & (0x80UL>>(column & 0x07UL)) )  /* calculate a shifted bit position based on the column */
				{
					*(destPtr + (column*rotWidthBytes) - (row >> 3)) |= (0x01UL<<(row & 0x07UL)); /* Set a bit based on the column */
				}
        }
    }

    /* clean up the used source BitMap, and restore the ports properly */
    DisposePtr(srcPort.portBits.baseAddr);
    ClosePort(&srcPort);
    SetPort(savePort);

    return noErr;
}